#include "stdafx.h"


//-- strings used for QubTree names when storing Idealizations 
const char * pchIdls = "Idls";
const char * pchIdl = "Idl";
const char * pchIdlSeg = "Segment";
const char * pchIdlOff = "Offset";
const char * pchIdlLen = "Length";
const char * pchIdlRGB = "RGB";
const char * pchIdlChn = "Channel";
const char * pchIdlAmp = "Amps";
const char * pchIdlStd = "StdDevs";
const char * pchIdlCls = "Classes";
const char * pchIdlDur = "Durations";
const char * pchIdlSta = "States";
const char * pchIdlFirsts = "Firsts";
const char * pchIdlLasts = "Lasts";


//-----------------------------------------------------------------------------
bool CDataFileInterface :: Open( const char * pchFileName, double * pdScaling, double * pdSampling, 
			CDataConfigure * pConfig, CItemArray * paSegments, CItemArray * paLists ) { 
	return true;
	} 
	
bool CDataFileInterface :: SaveData( const char * pchFileName, double dSampling, double dScaling, 
			CDataConfigure * pConfig, CItemArray * paSegments, CItemArray * paLists) {
	return true;
	} 

bool CDataFileInterface :: CreateData( const char * pchFileName, double dSampling, double dScaling, 
			CDataConfigure * pConfig, CItemArray * paSegments, CItemArray * paLists) {
	return true;
	} 
	
bool CDataFileInterface :: Close( ) { 
	return true;
	}

bool CDataFileInterface :: LoadSegmentData( CSegment * pSeg ){ 
	return true;
	}

bool CDataFileInterface :: SaveSegmentData( CSegment * pSeg ){ 
	return true;
	}


//-----------------------------------------------------------------------------
bool CQDSFInterface :: Open( const char * pchFileName, double * pdSampling, double * pdScaling, 
			CDataConfigure * pConfig, CItemArray * paSegments, CItemArray * paLists ) { 
	//-- close any previous file and clear data 
	Close();
	
	//-- Open the file 
	if( ! m_oQDSF.Open( pchFileName ) ) 
		return false;

	//-- Load Scaling
	//m_oQDSF.GoRoot();
	if( m_oQDSF.GoChild( "Scaling", false ) ) 
		m_oQDSF.m_oCurrent.DataBlock( m_oQDSF.m_hFile, pdScaling, 8 );
	else 
		*pdScaling=1.0;
	m_dFileScaling=*pdScaling;

	//-- Load Sampling
	m_oQDSF.GoRoot();
	if( m_oQDSF.GoChild( "Sampling", false ) ) 
		m_oQDSF.m_oCurrent.DataBlock( m_oQDSF.m_hFile, pdSampling, 8 );
	else 
		*pdSampling=0.001;

	//-- Set the filename 
	m_cFileName=pchFileName;

	//-- Load the channel info - does not return false on failure, just has no channels
	int iChannel=0;
	CChannel * pCh;
	m_oQDSF.GoRoot();		// unneeded
	if( m_oQDSF.GoChild( "SessionFile", false ) ) {
		if( m_oQDSF.GoChild( "DataChannels", false )
			&& m_oQDSF.GoChild() ) 
			do {
				if( m_oQDSF.m_oCurrent.m_cName == "Channel" && iChannel < MAX_CHANNELS ) { 
					//-- add or modify channel definition 
					if( m_oQDSF.GoChild()  ) { 
						if( iChannel>=pConfig->ChannelCount() ) 
							pConfig->AddChannel();
						pCh = pConfig->Channel(iChannel);
						do {	
							if( m_oQDSF.m_oCurrent.m_cName == "Name" )
								pCh->m_cName = m_oQDSF.CurrentString();
							else if( m_oQDSF.m_oCurrent.m_cName == "Units" ) 
								pCh->m_cUnits = m_oQDSF.CurrentString();
							else if( m_oQDSF.m_oCurrent.m_cName == "Scaling" ) 
								pCh->m_dChScaling = 1.0f/m_oQDSF.CurrentDouble();
							else if( m_oQDSF.m_oCurrent.m_cName == "Config" ) { 
								CString cData = m_oQDSF.CurrentString();
								pCh->FromString( cData.ConstBuffer() );
								}
							} while( m_oQDSF.GoSibling() );
						m_oQDSF.GoParent();		
						}
					++iChannel;
					}
				else if( m_oQDSF.m_oCurrent.m_cName == "Config" ) { 
					CString cConfig = m_oQDSF.m_oCurrent.DataString( m_oQDSF.m_hFile );
					pConfig->FromStringDataConfig( &cConfig );
					}
				} while( m_oQDSF.GoSibling( ) );
			
		}
	else if( m_oQDSF.GoRoot() && m_oQDSF.GoChild( "ADChannelCount", false ) ) {
		int iChannels = m_oQDSF.CurrentInt();
		while( iChannels > pConfig->ChannelCount() ) 
			pConfig->AddChannel();
		}

	//-- Load the data segment headers - does not return false on failure.
	m_oQDSF.GoRoot();
	if( m_oQDSF.GoChild( "Segments", false ) && m_oQDSF.GoChild("Segment",false) ) 
		do { 
			//-- Add a segment data header 
			if( m_oQDSF.GoChild() ) { 
				CSegment * pSeg = new CSegment;
				paSegments->AddItem( pSeg );
				do {	
					if( m_oQDSF.m_oCurrent.m_cName == "StartTime" ) 
						pSeg->m_dtStartTime = m_oQDSF.CurrentDouble();
					else if( m_oQDSF.m_oCurrent.m_cName == "Channels" ) {
						pSeg->m_iNodePos=m_oQDSF.m_oCurrent.m_iPos;
						pSeg->m_iPos=m_oQDSF.m_oCurrent.m_iDataPos;						
						pSeg->m_iSamples=m_oQDSF.m_oCurrent.m_iCount;
						pSeg->m_pdData=NULL;
						pSeg->m_bInterleaved=true;	// always for .qdf's
						pSeg->m_iChannels=pConfig->ChannelCount();		
						// TODO - if data segment is small and position is near current, load it now  ? 
						}
					else if( m_oQDSF.m_oCurrent.m_cName == "Digout" ) {
						pSeg->m_bDigout=true;
						int iDO = m_oQDSF.m_oCurrent.m_iCount;
						unsigned int * piDO = (unsigned int *)(m_oQDSF.GetDataBufferToDelete());
						pSeg->m_oDigoutList.LoadArray( iDO, piDO );
						delete [] piDO;
						}
					} while( m_oQDSF.GoSibling() );
				m_oQDSF.GoParent();		
				}
			} while( m_oQDSF.GoSibling("Segment") );
	
	CopyChannelScaling( &pConfig->m_aChannels );		
	
	//-- Load Idealizations 
	int iSegment, iCount, i, iFirsts=-1, iLasts=-1;
	BYTE * btBuf, * btFirsts=NULL, * btLasts=NULL;
	short bitStatus;			// set bits as each portion is loaded, accept if all bits are set, which indicates all pieces are loaded.
	if( m_oQDSF.GoRoot() 
			&& m_oQDSF.GoChild( "SessionFile", false ) 
			&& m_oQDSF.GoChild( pchIdls, false ) 
			&& m_oQDSF.GoChild( ) ) 
		do { 
			if( m_oQDSF.m_oCurrent.m_cName == pchIdl ) { 
				bitStatus=0;
				
				//-- create a IdlList to hold it 
				CIdlList * pIdlList = new CIdlList;
				if( m_oQDSF.GoChild() )
					do { 
						//-- read segment
						if( m_oQDSF.m_oCurrent.m_cName==pchIdlSeg ) {
							iSegment=m_oQDSF.CurrentInt();
							bitStatus|=1;
							}
						//-- Offset
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlOff ) {
							pIdlList->m_iOffset=m_oQDSF.CurrentInt();
							bitStatus|=2;
							}
						//-- Length 
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlLen ){
							pIdlList->m_iLength=m_oQDSF.CurrentInt();
							bitStatus|=4;
							}
						//-- Channel
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlChn ){
							pIdlList->m_iChannel=m_oQDSF.CurrentInt();
							bitStatus|=8;
							}
						//-- array of Channel Amps
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlAmp ){
							btBuf=m_oQDSF.GetDataBufferToDelete();
							iCount=m_oQDSF.m_oCurrent.m_iCount;
							pIdlList->AllocateClasses( iCount );
							for( i=0; i<iCount; ++i ) 
								pIdlList->IdlClass(i)->m_dAmp=((double *)btBuf)[i];
							delete [] btBuf;
							bitStatus|=16;
							}
						//-- array of channeld StdDev's
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlStd ){
							btBuf=m_oQDSF.GetDataBufferToDelete();
							iCount=m_oQDSF.m_oCurrent.m_iCount;
							pIdlList->AllocateClasses( iCount );
							for( i=0; i<iCount; ++i ) 
								pIdlList->IdlClass(i)->m_dStdDev=((double *)btBuf)[i];
							delete [] btBuf;
							bitStatus|=32;
							}
						//-- array of dwell classes 
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlCls ){
							btBuf=m_oQDSF.GetDataBufferToDelete();
							iCount=m_oQDSF.m_oCurrent.m_iCount;
							pIdlList->AllocateDwells( iCount );
							for( i=0; i<iCount; ++i ) 
								pIdlList->IdlDwell(i)->m_iClass=((short *)btBuf)[i];
							delete [] btBuf;
							bitStatus|=64;
							}

						//-- array of firsts and lasts
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlFirsts ){
							btFirsts=m_oQDSF.GetDataBufferToDelete();
							iFirsts=m_oQDSF.m_oCurrent.m_iCount;
							}
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlLasts ){
							btLasts=m_oQDSF.GetDataBufferToDelete();
							iLasts=m_oQDSF.m_oCurrent.m_iCount;
							}
							
						//-- array of dwell durations
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlDur ){
							btBuf=m_oQDSF.GetDataBufferToDelete();
							iCount=m_oQDSF.m_oCurrent.m_iCount;
							pIdlList->AllocateDwells( iCount );
							for( i=0; i<iCount; ++i ) 
								pIdlList->IdlDwell(i)->m_iDuration=int(((float *)btBuf)[i]/(*pdSampling)+0.5);
								
							delete [] btBuf;
							bitStatus|=128;
							}
							
						//-- array of dwell states
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlSta ){
							btBuf=m_oQDSF.GetDataBufferToDelete();
							iCount=m_oQDSF.m_oCurrent.m_iCount;
							pIdlList->AllocateDwells( iCount );
							for( i=0; i<iCount; ++i )
								pIdlList->IdlDwell(i)->m_iState=((short *)btBuf)[i];
							delete [] btBuf;
							bitStatus|=256;
							}
						//-- Color
						else if( m_oQDSF.m_oCurrent.m_cName==pchIdlRGB ){
							pIdlList->m_crColor=(COLORREF)m_oQDSF.CurrentInt();
							//optional, bitStatus unchanged
							}
							
						} while( m_oQDSF.GoSibling() );


				//-- if Firsts and Lasts are loaded, use them instead of durations
				if( btFirsts!=NULL && btLasts!=NULL && iFirsts==iLasts && iFirsts==pIdlList->m_aIdlDwells.Count() ) 
					for( int iDwell=0; iDwell<iFirsts; ++iDwell ) 
						pIdlList->IdlDwell(iDwell)->m_iDuration=((long *)btLasts)[iDwell]-((long *)btFirsts)[iDwell]+1;
				if( btFirsts!=NULL ) 
					delete [] btFirsts;
				btFirsts=NULL;
				if( btLasts!=NULL ) 
					delete [] btLasts;
				btLasts=NULL;
						
				//-- if all items ( bits 0-7 set ) were loaded, add to the correct idl array.
				if( bitStatus==511 ) 
					((CSegment *)(paSegments->GetAt( iSegment )) )->m_aIdl.AddItem( pIdlList );
				else 
					delete pIdlList;
				m_oQDSF.GoParent();
				}
				
			} while( m_oQDSF.GoSibling() ) ;
	else { 
		#ifdef _DEBUG
		//----- if no idealizations, try to load one from.qsf
		char buf[1024], * pExt;
		strcpy( buf, pchFileName );
		CQFFile o;
		if( (pExt=strstr( buf, ".qdf" ))!=NULL 
			&& sprintf( pExt,".qsf" ) > 0 
			&& FileExists( buf ) 
			&& o.Open(buf)
			&& o.GoRoot() 
			&& o.GoChild( "Idealization", false, QTP_EMPTY, 0, 0 ) 
			&& o.GoChild( "Channel", false, QTP_EMPTY, 0, 0 ) 
			&& o.GoChild( "Segment", false, QTP_LONG, 0, 0 ) ) { 
			
			o.GoChild();
			CIdlList * pIdlList = new CIdlList;
			bitStatus=0;
			do { 
				//-- list nodes names 
				//========================================================
				pIdlList->m_iChannel=0;				bitStatus|=8;
				iSegment=0;							bitStatus|=1;
				pIdlList->m_iOffset=0;				bitStatus|=2;
				pIdlList->m_iLength=0;				bitStatus|=4;

				//-- array of Channel Amps
				if( o.m_oCurrent.m_cName=="amp" ){
					btBuf=o.GetDataBufferToDelete();
					iCount=o.m_oCurrent.m_iCount;
					pIdlList->AllocateClasses( iCount );
					for( i=0; i<iCount; ++i ) 
						pIdlList->IdlClass(i)->m_dAmp=((double *)btBuf)[i];
					delete [] btBuf;
					bitStatus|=16;
					}
				//-- array of channeld StdDev's
				else if( o.m_oCurrent.m_cName=="sd" ){
					btBuf=o.GetDataBufferToDelete();
					iCount=o.m_oCurrent.m_iCount;
					pIdlList->AllocateClasses( iCount );
					for( i=0; i<iCount; ++i ) 
						pIdlList->IdlClass(i)->m_dStdDev=((double *)btBuf)[i];
					delete [] btBuf;
					bitStatus|=32;
					}
				//-- array of dwell classes 
				else if( o.m_oCurrent.m_cName==pchIdlCls ){
					btBuf=o.GetDataBufferToDelete();
					iCount=o.m_oCurrent.m_iCount;
					pIdlList->AllocateDwells( iCount );
					for( i=0; i<iCount; ++i ) 
						pIdlList->IdlDwell(i)->m_iClass=((int *)btBuf)[i];
					delete [] btBuf;
					bitStatus|=64;
					}

				//-- array of firsts and lasts
				else if( o.m_oCurrent.m_cName==pchIdlFirsts ){
					btFirsts=o.GetDataBufferToDelete();
					iFirsts=o.m_oCurrent.m_iCount;
					}
				else if( o.m_oCurrent.m_cName==pchIdlLasts ){
					btLasts=o.GetDataBufferToDelete();
					iLasts=o.m_oCurrent.m_iCount;
					}
					
				//-- array of dwell durations
				else if( o.m_oCurrent.m_cName==pchIdlDur ){
					btBuf=o.GetDataBufferToDelete();
					iCount=o.m_oCurrent.m_iCount;
					pIdlList->AllocateDwells( iCount );
					for( i=0; i<iCount; ++i ) {
						pIdlList->IdlDwell(i)->m_iDuration=int(((float *)btBuf)[i]/(*pdSampling)+0.5);
						pIdlList->m_iLength += pIdlList->IdlDwell(i)->m_iDuration;
						}
						
					delete [] btBuf;
					bitStatus|=128;
					}
					
				} while( o.GoSibling() );


			//-- if Firsts and Lasts are loaded, use them instead of durations
			if( btFirsts!=NULL && btLasts!=NULL && iFirsts==iLasts && iFirsts==pIdlList->m_aIdlDwells.Count() ) {
				for( int iDwell=0; iDwell<iFirsts; ++iDwell ) 
					pIdlList->IdlDwell(iDwell)->m_iDuration=((long *)btLasts)[iDwell]-((long *)btFirsts)[iDwell]+1;
				pIdlList->m_iLength = ((long *)btLasts)[iLasts-1];
				}
			if( btFirsts!=NULL ) 
				delete [] btFirsts;
			btFirsts=NULL;
			if( btLasts!=NULL ) 
				delete [] btLasts;
			btLasts=NULL;
					
			//-- if all items ( bits 0-7 set ) were loaded, add to the correct idl array.
			if( bitStatus==255 ) 
				((CSegment *)(paSegments->GetAt( iSegment )) )->m_aIdl.AddItem( pIdlList );
			else 
				delete pIdlList;
			o.GoParent();
			//========================================================
			
			o.Close();
			}
		#endif
		}
	
	//----- Load Lists 
	if( m_oQDSF.GoRoot()
				&& m_oQDSF.GoChild( "SessionFile", false ) 
				&& m_oQDSF.GoChild("Lists") 
				&& m_oQDSF.GoChild() ) { 
		int aiListItem[2], iSeg0, iSeg1, iPt0, iPt1;
		CString cLabel, cComment;
		bool bComment;
		
		do {										//-- for each child
			if( m_oQDSF.m_oCurrent.m_cName=="List" ) { 
				CSelList * pList = new CSelList( m_oQDSF.m_oCurrent.DataString(m_oQDSF.m_hFile).ConstBuffer() );
				if( m_oQDSF.GoChild() ) { 			//-- down to 1st list item 
					do {							//-- for each list item 
						if( ! ( m_oQDSF.m_oCurrent.m_cName=="Sels" ) ) { 
							m_oQDSF.m_oCurrent.DataBlock( m_oQDSF.m_hFile, aiListItem, 8, 0 );
							
							//-- Convert point index to segment/offet 
							//PointToSegOffset( aiListItem[0], &iSeg0, &iPt0 );
							//PointToSegOffset( aiListItem[1], &iSeg1, &iPt1 );
							iSeg0=iSeg1=-1;
							iPt0=iPt1=0;
							int iPts=0;
							for( int iSeg=0; iSeg<paSegments->Count() && ( iSeg0=-1 || iSeg1==-1 ) ; ++iSeg ) { 
								CSegment * pSeg = (CSegment *)(paSegments->GetAt(iSeg));
								if( iPts+pSeg->m_iSamples > aiListItem[0] ) { 
									iSeg0=iSeg;
									iPt0=aiListItem[0]-iPts;
									}
								if( iPts+pSeg->m_iSamples > aiListItem[1] ) { 
									iSeg1=iSeg;
									iPt1=aiListItem[1]-iPts;
									}
								iPts+=pSeg->m_iSamples;
								}
							if( iSeg0==-1 || iSeg1==-1 ) 
								iSeg0=iSeg1=0;
							
							iPt1++;
							cLabel = m_oQDSF.m_oCurrent.m_cName;
							if( m_oQDSF.GoChild() && m_oQDSF.m_oCurrent.m_cName=="Comment" ) { 
								cComment = m_oQDSF.m_oCurrent.DataString(m_oQDSF.m_hFile);
								m_oQDSF.GoParent();
								bComment=true;
								}
							else 
								bComment=false;
							pList->AddListItem( iSeg0, iPt0, iSeg1, iPt1, cLabel.ConstBuffer(), bComment?cComment.ConstBuffer():NULL );
							}
						} while( m_oQDSF.GoSibling() );	//-- next list item
					m_oQDSF.GoParent();
					}
				paLists->AddItem( pList );
				} 
			} while( m_oQDSF.GoSibling() );		//-- next list 
		}

	//----- Load Cursors
	//TODO
	
	//----- Load Fit Curves
	// TODO 
	
	// done.   The file tree structure is loaded, but the actual data may not yet be.
	return true;
	}
	
bool CQDSFInterface :: CalculateScaling( CItemArray * paChannels, CItemArray * paSegments, double dScaling ) { 
	//-- Calculate short integer scaling factors based on max data value in each channel 
	int iCh, iChannels=paChannels->Count();
	int iSeg, iSegments=paSegments->Count();
	int iPt, iIndex, iSep;
	double dAbsMax;

	m_adScaling = new double[iChannels];
	if( m_adScaling == NULL )
		return false;
		
	for( iCh=0; iCh<iChannels; ++iCh ) {
		CChannel * pCh = ((CChannel *)paChannels->GetAt(iCh));
		dAbsMax=0.0;
		for( iSeg=0; iSeg<iSegments; ++iSeg ) {
			CSegment * pSeg = (CSegment *)paSegments->GetAt(iSeg);
			iIndex = pSeg->m_bInterleaved ?  iCh : iCh*pSeg->m_iSamples;
			iSep = pSeg->m_bInterleaved ? iChannels : 1;
			for( iPt=0; iPt<pSeg->m_iSamples; ++iPt, iIndex+=iSep ) 
				dAbsMax=max( dAbsMax, fabs( pSeg->m_pdData[iIndex] ) );
			}
		pCh->m_dChScaling = m_adScaling[iCh] = double(0x7FFE)/dAbsMax/dScaling;
		}
	return true;
	}

void CQDSFInterface :: CopyChannelScaling( CItemArray * paChannels ) { 
	//-- Copy channel scaling factors to an array for quick access 
	int iChannels=paChannels->Count();
	m_adScaling = new double[iChannels];
	if( m_adScaling != NULL ) 
		for( int iCh=0; iCh<iChannels; ++iCh )
			m_adScaling[iCh]=((CChannel *)paChannels->GetAt(iCh))->m_dChScaling;
	}

//----- 
bool CQDSFInterface :: Close( ) { 
	if( ! m_oQDSF.Opened() ) 
		return false;
	m_oQDSF.Close();

	if( m_adScaling!=NULL ) {
		delete [] m_adScaling;
		m_adScaling=NULL;
		}
	return true;
	}	

//--
bool CQDSFInterface :: LoadSegmentData( CSegment * pSeg ){ 
	//-- check if already loaded 
	if( pSeg->m_pdData != NULL ) 
		return true;
		
	//-- Position file pointer
	if( fseek( m_oQDSF.m_hFile, pSeg->m_iPos, SEEK_SET ) != 0 ) 
		return false;
		
	//-- Load data block 
	//TODO : check for file type and load integers or doubles as appropriate.
	return pSeg->LoadDataShort( pSeg->m_iSamples, pSeg->m_iChannels, m_oQDSF.m_hFile, m_adScaling, m_dFileScaling  );
	}

//--
bool CQDSFInterface :: SaveSegmentData( CSegment * pSeg ) { 
	if( pSeg->m_iPos==0 ) {
		//TODO - add new segment and set iPos 
		}
	//TODO - copy data to disk file ( this->m_oQDSF.m_hFile ) 
	// This should be made thread safe - possibly via separate thread dedicated to each datafile
	
	return true;
	}

//-- Create a new .QDF (with sessionfile internal) 
bool CQDSFInterface :: CreateData( const char * pchFileName, double dSampling, double dScaling,
			CDataConfigure * pConfig, CItemArray * paSegments, CItemArray * paLists) { 
	int iChannels=pConfig->ChannelCount();
	int * aiSegOffsets = new int[paSegments->Count()+1];	// allocate+1 for simpler running total 
	
	bool bResult = 	
		CalculateScaling( &pConfig->m_aChannels, paSegments, dScaling )					//-- fill in m_adScaling
		&& m_oQDSF.Create( pchFileName, "DataFile" )							//-- DataFile
		&& m_oQDSF.WriteHeader( dSampling, dScaling, iChannels, 2, QTP_SHORT )	//-- .Sampling, .Scaling, .ADChannelCount, .ADDataSize, .ADDataType			
		&& m_oQDSF.WriteSegments( iChannels, m_adScaling, paSegments, aiSegOffsets, dScaling ) //-- Segments (data) 
		&& m_oQDSF.AddNode( "SessionFile", true, QTP_EMPTY, 0, 0 )				//--.SessionFile
		&& m_oQDSF.WriteComment( "GEM file" )									//-- ..Comment
		&& m_oQDSF.WriteChannels( pConfig )										//-- ..DataChannels
		&& m_oQDSF.WriteLists( paLists, aiSegOffsets )							//-- ..Lists, ...List, ....(Item Label), .....Comment
		&& m_oQDSF.WriteIDL( &pConfig->m_aChannels, paSegments, aiSegOffsets, dSampling ); //-- ..Idealization ...Channel ....sampling ( etc. )
	if( ! bResult ) 
		MessageBox( NULL, "Error saving file", pchFileName, MB_OK );
	delete [] aiSegOffsets;
	return bResult;
	}			

//-- update an existing .qdf file 
bool CQDSFInterface :: SaveData( const char * pchFileName, double dSampling, double dScaling,
			CDataConfigure * pConfig, CItemArray * paSegments, CItemArray * paLists) { 
	int iChannels=pConfig->ChannelCount();
	bool bResult = false;

	//-- empty SessionFile node and refill it.
	if( ! m_oQDSF.GoRoot()
		|| ! m_oQDSF.GoChild( "SessionFile" ) 
		|| ! m_oQDSF.DeleteNode() 
		|| ! m_oQDSF.GoRoot()
		|| ! m_oQDSF.GoChild( "SessionFile", true ) )
		return false;

	//-- create segments offset array 		
	int iOffset=0, iSegments = paSegments->Count();
	int * aiSegOffsets = new int[iSegments+1];
	for( int iSeg=0; iSeg<iSegments; ++iSeg ) {
		aiSegOffsets[iSeg]=iOffset;
		iOffset += ((CSegment *)paSegments->GetAt(iSeg))->m_iSamples;
		}
	aiSegOffsets[iSegments]=iOffset;
	
	//-- write updated contents
	bResult = m_oQDSF.WriteComment( "GEM file" )								//-- ..Comment
		&& m_oQDSF.WriteChannels( pConfig )										//-- ..DataChannels
		&& m_oQDSF.WriteLists( paLists, aiSegOffsets )							//-- ..Lists, ...List, ....(Item Label), .....Comment
		&& m_oQDSF.WriteIDL( &pConfig->m_aChannels, paSegments, aiSegOffsets, dSampling )	//-- ..Idealization ...Channel ....sampling ( etc. )
		;
	delete [] aiSegOffsets;

	if( ! bResult ) 
		MessageBox( NULL, "Error saving file", pchFileName, MB_OK );
		
	return bResult;
	}			
